#include "escena.h"



Escena::Escena(QWidget *parent)
: QGLWidget(parent)
{
    int cols = 640;
    int rows = 480;
	int deviceID = 0;       // 0 = open default camera
    int apiID = 1;//cv::CAP_ANY; 

    setFocusPolicy(Qt::StrongFocus);
    timer = new QTimer(this);

    connect(timer, SIGNAL(timeout()), this, SLOT(timerDone()));

    //setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    //Initiating camera
    cap = new VideoCapture();
	cap->open( deviceID + apiID ); // open the  camera
	
    if (!cap->isOpened()) { // check if we succeeded
        printf("Error while opening camera device cap(1)\n");
        exit(0);
    }

    IMG = matrix_allocate( rows, cols, sizeof (char));
    if (!IMG) {
        fprintf(stderr, "ERROR: No memory\n");
        exit(0);
    }

    cap->set(CV_CAP_PROP_FRAME_WIDTH, cols);
    cap->set(CV_CAP_PROP_FRAME_HEIGHT, rows);
    // namedWindow("Bordes", 1);

    *(cap) >> frame; // get a new frame from camera
    //imwrite( "imagen.png", frame );

    cv::cvtColor(frame, grayFrame, CV_BGR2GRAY);

    // IMG->rows = grayFrame.rows;
    // IMG->cols = grayFrame.cols;
	// fprintf( stderr, "%d %d\n", grayFrame.rows, grayFrame.cols );
	// fprintf( stderr, "%d %d\n", IMG->rows, IMG->cols );
    IMG->type = 5;
    IMG->element_size = sizeof (char);

    
    //Set default points
    T1[0].x = 0;    T1[0].y = 0;
    T1[1].x = 320;    T1[1].y = 0;
    T1[2].x = 320;    T1[2].y = 240;
    
    T2[0].x = 640;    T2[0].y = 480;
    T2[1].x = 320;    T2[1].y = 480;
    T2[2].x = 320;    T2[2].y = 240; 


    //Start Camera by default
    initscreen();

}

void Escena::timerDone() {
    
    find_TRIANGLES( IMG , T1 ,T2 );

    updateGL();
}


void Escena::paintGL(void) {
    draw_IMG();

    draw_Triangles();
}

void Escena::draw_IMG() {
    glDisable(GL_DEPTH_TEST);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //getting image from camera
    *(cap) >> frame; // get a new frame from camera
    //Preprocessing image
    //Thresholding settings
    //int threshold_value = 100;
    //int threshold_type = 1;
    //int const max_BINARY_value = 255;

    flip(frame, frame, 0);
    cv::cvtColor(frame, grayFrame, CV_BGR2GRAY);
    // GaussianBlur(grayFrame, grayFrame, Size(7, 7), 1.5, 1.5); //Smooth filter
    // threshold(grayFrame, grayFrame, threshold_value, max_BINARY_value, threshold_type);
    // bitwise_not(grayFrame, grayFrame);

    unsigned char b;
    int channel = 1;
    unsigned char *input = (unsigned char*) (grayFrame.data);
    for (int i = 0; i < grayFrame.rows; i++) {
        for (int j = 0; j < grayFrame.cols; j++) {
            b = input[grayFrame.step * i + j * channel ];
            ((byte**) IMG->ptr)[i][j] = b;
        }
    }

    //IMG is the camera frame in graytones, and can be processed after this line


    //use fast 4-byte alignment (default anyway) if possible
    glPixelStorei(GL_UNPACK_ALIGNMENT, (grayFrame.step & 3) ? 1 : 4);
    //set length of one complete row in data (doesn't need to equal image.cols)
    // glPixelStorei(GL_UNPACK_ROW_LENGTH, grayFrame.step / grayFrame.elemSize());
    //flip(frame, frame, 0);
    //Drawing color image to the QGL canvas
    // glDrawPixels(frame.cols, frame.rows, GL_BGR, GL_UNSIGNED_BYTE, grayFrame.data);
    glDrawPixels(frame.cols, frame.rows, GL_LUMINANCE, GL_UNSIGNED_BYTE, grayFrame.data);
    glEnable(GL_DEPTH_TEST);
}

void drawSquare()
{
    glBegin( GL_QUADS );
        glVertex2i( -1 , -1 );
        glVertex2i( 1  , -1 );
        glVertex2i( 1  ,  1 );
        glVertex2i( -1 ,  1 );
    glEnd();
}

void DRAW1()
{   
    float angle = 45;
    int cross = 1;

     //Draw
                    
        //printf("windowWidth = %d \n", windowWidth );
        //printf("windowHeight= %d \n", windowHeight );
        glPushMatrix( );
                    //glTranslatef( windowWidth/2, windowHeight/2 , 0.0 );
                    glTranslatef( 640/2, 480/2 , 0.0 );

                    glRotatef ( 90 , 0.0, 0.0, 1.0);

                    float SS = 25.0;
                    glScalef( SS, SS, SS );

                    glPushMatrix( ); //BIG Square
                        //glRotatef ( angle , 0.0, 0.0, 1.0);
                        //glTranslatef( 1.0, 1.0, 0.0 );
                        glColor3f( 0.4 , 0.4, 0.4);
                        drawSquare( );
                    glPopMatrix( );

                    glColor3f( 0.44 , 0.8, 0.7);
                    glPushMatrix( ); // LittleSquare 1
                        //glColor3f( 1.0 , 0.8, 0.4);
                        glTranslatef(-1.5, 1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle*cross , 0.0, 0.0, 1.0); //Cross Spin
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 2
                        //glColor3f( 1.0 , 0.4, 0.4);
                        glTranslatef(1.5, 1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle , 0.0, 0.0, 1.0);
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 3
                        //glColor3f( 1.0 , 0.4, 1.0);
                        glTranslatef(-1.5, -1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle , 0.0, 0.0, 1.0);
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 4
                        //glColor3f( 1.0 , 0.4, 0.4);
                        glTranslatef(1.5, -1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle*cross , 0.0, 0.0, 1.0); //Cross Spin
                        drawSquare( );
                    glPopMatrix( );
        glPopMatrix( );
}

void Escena::draw_Triangles() 
{
    /** Clear all pixels **/
    //glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glViewport (0, 0, (GLsizei) 640, (GLsizei) 480);

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    glOrtho( 0, 640, 
             0, 480, 
            -1.0, 1.0 );

   
    float SS = 50.0;
   

   //Draw triangle1
    glPushMatrix( );
            //glTranslatef( 640/2, 480/2 , 0.0 );
            //glScalef( SS, SS, SS );

            glBegin(GL_LINE_LOOP); 
                glVertex3f( T1[0].x,  T1[0].y,  0);
                glVertex3f( T1[1].x,  T1[1].y,  0);
                glVertex3f( T1[2].x,  T1[2].y,  0);
                glVertex3f( T1[0].x,  T1[0].y,  0);
            glEnd();
       
    glPopMatrix( );




    //Draw triangle2
    glPushMatrix( );
        glBegin(GL_LINE_LOOP); 
            glVertex3f( T2[0].x,  T2[0].y,  0);
            glVertex3f( T2[1].x,  T2[1].y,  0);
            glVertex3f( T2[2].x,  T2[2].y,  0);
            glVertex3f( T2[0].x,  T2[0].y,  0);
        glEnd();
    glPopMatrix( );




    //Reset
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity ();
}



void Escena::mouseMoveEvent(QMouseEvent * e) {
    //Not implemented
    return;
}

void Escena::keyPressEvent(QKeyEvent * e) {
    //Not implemented
    return;
}

//
// This method puts the initial coordinates each time window is resized
//

void Escena::resizeGL(int l, int a) {
    windowWidth = l;
    windowHeight = a;
    glViewport(0, 0, (GLsizei) l, (GLsizei) a);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    //emit changeSize();
}

QSizePolicy Escena::sizePolicy() const {
    return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}


void Escena::initializeGL() {
    glClearColor(0.8, 0.8, 0.8, 0.0); // Background to a grey tone
}

//--------------------------------------
//--------------------------------------


void Escena::capture()
{
    write_pnm( IMG, "inputIMG.pgm" );
    printf("CAPTURED!\n");

    //smoothIMG( IMG );
    //smoothIMG( IMG );
    //smoothIMG( IMG );

    //write_pnm( IMG, "inputIMGS.pgm" );

    find_object( IMG );
    find_TRIANGLES( IMG , T1 ,T2 );

   



    //Send message to GUI
    emit trainMsg();
}



void Escena::classify()
{

}





